"""
A dtddoc backend that produces HTML output.

$Id: htmldoc.py,v 1.8 2000/09/25 14:34:27 larsga Exp $
"""

import types,sys

if hasattr(types, "UnicodeType"): # hack to make it work with Py 1.5.x and 2.0
    utype = types.UnicodeType
else:
    utype = types.StringType # just a dummy value

import docutil

def elem_ref_fun(name,modifier):
    if name == "#PCDATA":
        return name
    else:
        return '<A HREF="#%s">%s</A>%s' % (name,name,modifier)

def write_descr(out,descr,wrapper="P"):
    out.write("<%s>\n" % wrapper)
    for item in descr:
        if type(item) == types.StringType or type(item) == utype:
            out.write(item)
        else:
            # can currently only be elemref
            (elem,text)=item
            out.write('<A HREF="#%s">%s</A>' % (text,text))
    out.write("\n</%s>\n" % wrapper)    

def creator_tohtml(creator):
    if creator.homep:
        html = '<a href="%s">%s</a>' % (creator.homep,creator.name)
    else:
        html = creator.name

    if creator.email:
        html = '%s - <a href="mailto:%s">%s</a>' % (html, creator.email,
                                                    creator.email)

    return html
    
def write_doco(out,dtddoc,dtd,parents,options):
    css=""
    if options.css:
        css='<LINK REL=stylesheet HREF="%s" TYPE="text/css">' % options.css
    
    out.write(
    """
    <HTML>
    <HEAD>
      <TITLE>Documentation for %s</TITLE>
      %s
    </HEAD>

    <BODY>
    <H1>Documentation for %s</H1>
    """ % (dtddoc.name,css,dtddoc.name))

    out.write("<TABLE>\n")
    if dtddoc.version:
        out.write("<TR><TH>Version:  <TD>%s\n" % dtddoc.version)
    for creator in dtddoc.creators:
        out.write("<TR><TH>Creator:  <TD>%s\n" % creator_tohtml(creator))
    if dtddoc.pubid:
        out.write("<TR><TH>PublicId: <TD><tt>%s</tt>\n" % dtddoc.pubid)
    out.write("</TABLE>\n")

    out.write("\n<H2>Description</H2>\n")
    write_descr(out,dtddoc.descr)

    if dtddoc.root_elems!=[]:
        out.write("<H2>Root elements</H2>\n")
        out.write("<UL>\n")
        for element in dtddoc.root_elems:
            out.write('  <LI><A HREF="#%s">%s</A>' % (element,element))
        out.write("</UL>\n")


    elems=dtd.get_elements()
    elems.sort()

    # Element index

    if options.elem_index:
        out.write("<H2>Element index</H2>\n")

        if options.elem_index_one_line:
            out.write("<TABLE>\n")
            for elem_name in elems:
                if dtddoc.elems.has_key(elem_name):
                    one_line=dtddoc.elems[elem_name].one_line
                else:
                    one_line=""
                out.write('<TR><TD><A HREF="#%s">%s</A> <TD>%s\n' %
                          (elem_name,elem_name,one_line))
            out.write("</TABLE>\n")
        else:
            out.write("<UL>\n")
            for elem_name in elems:
                out.write('  <LI><A HREF="#%s">%s</A>\n' %
                          (elem_name,elem_name))
            out.write("</UL>\n")            

    # Attribute index

    if options.attr_index:
        attrs=[]
        for elem_name in elems:
            elem=dtd.get_elem(elem_name)
            for attr_name in elem.get_attr_list():
                attrs.append((attr_name,elem_name))

        if attrs!=[]:                
            attrs.sort()

            out.write("<H2>Attribute index</H2>\n")

            out.write("<UL>\n")
            for (attr_name,elem_name) in attrs:
                out.write('  <LI><A HREF="#%s">%s</A> on %s\n' % \
                          (elem_name,attr_name,elem_name))
            out.write("</UL>\n")

    # Element descriptions

    out.write("<H2>Element descriptions</H2>\n")
    for elem_name in elems:
        out.write('<H3><A NAME="%s">%s</A></H3>' % (elem_name,elem_name))

        if dtddoc.elems.has_key(elem_name):
            write_descr(out,dtddoc.elems[elem_name].descr)

        out.write("<H4>Parents</H4>")

        out.write("<P>")
        ps=parents[elem_name].keys()
        if ps==[]:
            out.write("Must appear as the root element.\n")
        else:
            ps.sort()
            for p in ps:
                out.write('<A HREF="#%s">%s</A> ' % (p,p))

            out.write("</P>")

        elem=dtd.get_elem(elem_name)

        out.write("<H4>Content model</H4>")

        docutil.print_cm(out,elem.get_content_model(),elem_ref_fun)

        attrs=elem.get_attr_list()
        if attrs!=[] and dtddoc.elems.has_key(elem_name):
            out.write("<H4>Attributes</H4>")
            out.write("<TABLE>")
            out.write("<TR><TH>Name <TH>Type <TH>Declaration <TH>Default <TH>Description")

            attrs.sort()
            elem_doc=dtddoc.elems[elem_name]
            for attr_name in attrs:
                attr=elem.get_attr(attr_name)
                attr_doc=elem_doc.get_attribute(attr_name)

                if attr.get_default()!=None:
                    default=attr.get_default()
                else:
                    default=""

                out.write("  <TR><TD>%s <TD>%s <TD>%s <TD>%s " %
                          (attr_name,attr.get_type(),attr.get_decl(),
                           default))
                write_descr(out,attr_doc.descr,"TD")
            out.write("</TABLE>")   

    out.write(
    """
    <HR>

    <ADDRESS>
    Produced by <A HREF="http://www.stud.ifi.uio.no/~larsga/download/python/xml/dtddoc.html">dtddoc.py</A>, using xmlproc.
    </ADDRESS>

    </BODY>
    </HTML>
    """)

    out.close()
