"""
A dtddoc backend that produces DocBook RefEntry output.

$Id: refentry.py,v 1.4 2000/09/25 14:35:24 larsga Exp $
"""

import sys,string

import docutil

def elem_ref_fun(name,modifier):
    if name == "#PCDATA":
        return name
    else:
        return '<XRef Linkend="%s"/> %s' % (name,modifier)

def write_descr(out, descr):
    out.write("<Para>\n")
    out.write(descr[0])
    if len(descr) > 1:
        out.write(descr[1])    
    out.write("</Para>\n")

def write_doco(out,dtddoc,dtd,parenthash,options):
    if not options.silent:
        print "WARNING: This backend is _really_ experimental."
    
    out.write(
    """
    <!DOCTYPE Reference PUBLIC "-//Davenport//DTD DocBook V3.0//EN" "dbxml.dtd">
    <Reference>
      <Title>Documentation for %s</Title>
      
    """ % (dtddoc.name))

    # Introductory meta-info

    out.write('<PartIntro>\n')
    write_descr(out, dtddoc.descr)
    out.write('</PartIntro>\n')

    # Element descriptions

    for elem in dtddoc.get_elements():
        elem_def = dtd.get_elem(elem.name)
        
        parents = []
        ps=parenthash[elem.name].keys()
        if ps==[]:
            parents.append("Must appear as the root element.")
        else:
            ps.sort()
            for p in ps:
                parents.append('<XRef Linkend="%s"/>' % p)
        
        out.write('<RefEntry id="%s">\n' % elem.name)
        out.write('  <RefMeta>\n')
        out.write('    <RefEntryTitle>%s</RefEntryTitle>\n' % elem.name)
        out.write('  </RefMeta>\n')
        
        out.write('  <RefNameDiv>\n')
        out.write('    <RefName>%s</RefName>\n' % elem.name)
        out.write('    <RefPurpose>%s</RefPurpose>\n' % elem.descr[0])
        out.write('  </RefNameDiv>\n')
        
        out.write('  <RefSect1>\n')
        out.write('    <Title>Description</Title>\n')
        write_descr(out, elem.descr)
        out.write('  </RefSect1>\n')
        
        out.write('  <RefSect1>\n')
        out.write('    <Title>Parents</Title>\n')        
        out.write('    <Para>%s</Para>\n' % string.join(parents," | "))
        out.write('  </RefSect1>\n')

        out.write('  <RefSect1>\n')
        out.write('    <Title>Content model</Title>\n')
        out.write('    <Para>\n')
        docutil.print_cm(out,elem_def.get_content_model(),elem_ref_fun)
        out.write('</Para>\n')
        out.write('  </RefSect1>\n')

        out.write('  <RefSect1>\n')
        out.write('    <Title>Attributes</Title>\n')
        out.write('    <Para>No support for this yet.</Para>\n')
#         attrs=elem.get_attr_list()
#         if attrs!=[] and dtddoc.elems.has_key(elem_name):
#             out.write("<H4>Attributes</H4>")
#             out.write("<TABLE>")
#             out.write("<TR><TH>Name <TH>Type <TH>Declaration <TH>Default <TH>Description")

#             attrs.sort()
#             elem_doc=dtddoc.elems[elem_name]
#             for attr_name in attrs:
#                 attr=elem.get_attr(attr_name)
#                 attr_doc=elem_doc.get_attribute(attr_name)

#                 if attr.get_default()!=None:
#                     default=attr.get_default()
#                 else:
#                     default=""

#                 out.write("  <TR><TD>%s <TD>%s <TD>%s <TD>%s " %
#                           (attr_name,attr.get_type(),attr.get_decl(),
#                            default))
#                 write_descr(out,attr_doc.descr,"TD")
#             out.write("</TABLE>")   

        out.write('  </RefSect1>\n')
        
        out.write('</RefEntry>\n')

    # Closing
            
    out.write(
    """
    </Reference>
    """)

    out.close()
